from fastapi import FastAPI, Query, Body
from pydantic import BaseModel
import inspect
from typing import Any, get_type_hints

app = FastAPI()

def api_func(package_name: str):
    def wrapper(func):
        sig = inspect.signature(func)
        params = get_type_hints(func)

        # Sprawdzamy, czy funkcja wymaga JSON (czyli BaseModel → POST)
        is_post = any(
            isinstance(v, type) and issubclass(v, BaseModel) for v in params.values()
        )

        if is_post:
            param_name = next(iter(params))  # Pobierz nazwę parametru (np. "data")
            param_type = params[param_name]  # Pobierz typ parametru (BaseModel)

            async def endpoint_func(data: param_type = Body(...)):
                return {"result": func(data)}

            app.post(f"/{package_name}/{func.__name__}")(endpoint_func)

        else:
            # Tworzymy dynamiczne parametry jako Query
            async def endpoint_func(**query_params: Any):
                return {"result": func(**query_params)}

            # Modyfikacja sygnatury endpoint_func aby uwzględniać Query(...)
            new_params = []
            for name, param in sig.parameters.items():
                if name == "return":
                    continue
                new_params.append(
                    param.replace(default=Query(...))
                )
            new_sig = inspect.Signature(parameters=new_params)
            endpoint_func.__signature__ = new_sig

            # Rejestrujemy poprawnie GET w FastAPI
            app.get(f"/{package_name}/{func.__name__}")(endpoint_func)

        return func
    return wrapper
